#include <windows.h>
#include <stdio.h>

//#define STARTF_USEHOTKEY		0x200
#define STARTF_ICON				0x400
#define STARTF_MONITOR			0x400
#define STARTF_TITLESHORTCUT	0x800
#define STARTF_SCREENSAVER		0x80000000

// Returns TRUE if started through a shortcut, FALSE if not. 
// Also returns the name of the shortcut file (if any)
BOOL GetShortcutName(TCHAR *szLinkName, UINT nLinkNameSize)
{    
    STARTUPINFO si = { sizeof(si) };

    GetStartupInfo(&si);

    if(si.dwFlags & STARTF_TITLESHORTCUT)
    {
        lstrcpyn(szLinkName, si.lpTitle, nLinkNameSize);
        return TRUE;
    }

    return FALSE;
}

//
//	Start solitaire on specified monitor
//
BOOL CALLBACK EnumMonitorProc(HMONITOR hMonitor, HDC hdc, LPRECT rect, LPARAM data)
{
    STARTUPINFO          si = { sizeof(si) };
    PROCESS_INFORMATION  pi;
    char                 szExe[] = "sol.exe";

    // specify which monitor to use
    si.dwFlags     = STARTF_MONITOR;
    si.hStdOutput  = hMonitor;

    // start solitaire on specified monitor
    if(CreateProcess(0, szExe, 0, 0, FALSE, 0, 0, 0, &si, &pi))
    {
       CloseHandle(pi.hProcess);
       CloseHandle(pi.hThread);
    }

	return TRUE;
}

// call EnumMonitorProc (above) for every installed monitor on the system
void MonitorDemo()
{
    EnumDisplayMonitors(NULL, NULL, EnumMonitorProc, 0);
}

BOOL SpawnWithData()
{
	STARTUPINFO          si  = { sizeof(si) };
    PROCESS_INFORMATION  pi;

	char buf[200];
	char exe[200];

	GetStartupInfo(&si);
	
	// if this is the first time then respawn ourselves with
	// a new lpReserved2 buffer
	if(si.lpReserved2 == NULL)
	{
		*(DWORD *)buf = 0;
		lstrcpy(buf+sizeof(DWORD), "Hello using lpReserved2!!");

		// setup the buffer to pass to child process
		si.lpReserved2 = buf;
		si.cbReserved2 = sizeof(buf);

		GetModuleFileName(0, exe, 200);
    
		// create a new process with this data
		if(CreateProcess(0, exe, 0, 0, 0, 0, 0, 0, &si, &pi))
		{
	        CloseHandle(pi.hProcess);
		    CloseHandle(pi.hThread);
		}

		return TRUE;
	}
	// lpReserved has been specified so we are the spawned
	// child process - 
	else
	{
		MessageBox(0, si.lpReserved2+sizeof(DWORD), "lpReserved2", MB_OK);
		return FALSE;
	}
}

int main()
{
	char buf[200];

	//
	//	Demonstrate the lpReserved2 feature
	//
	if(!SpawnWithData())
		return 0;

	//
	// detect if started through a shortcut
	//
	if(GetShortcutName(buf, sizeof(buf)))
	{
		printf("Started via shortcut: %s\n", buf);
	}

	//
	// demonstrate the STARTF_MONITOR flag
	//
	MonitorDemo();

	return 0;
}
